using System;
using System.Collections.Generic;
using MacGruber;
using SimpleJSON;
using UnityEngine;

namespace CheesyFX
{
    public class CustomFloatTrigger : MonoBehaviour
    {
        private MVRScript script;
        public new string name;
        private string region;
        private JSONStorableFloat driver;
        public EventTrigger exceededTrigger;
        public EventTrigger undershootTrigger;
        public FloatTrigger valueTrigger;
        public JSONStorableFloat inputTo;
        public JSONStorableFloat inputFrom;
        // public JSONStorableFloat outputFrom;
        // public JSONStorableFloat outputTo;
        public JSONStorableFloat threshold;
        private bool panelOpen;
        private JSONStorableString info = new JSONStorableString("info", "");

        private float _lastValue;

        public void Init(MVRScript script, JSONStorableFloat driver, string region)
        {
            this.script = script;
            this.driver = driver;
            this.name = driver.name+"Trigger";
            this.region = region;
            inputFrom = new JSONStorableFloat("Input From", 0f, 0f, 1f, false);
            inputTo = new JSONStorableFloat("Input To", .25f, 0f, 1f, false);
            // outputFrom = new JSONStorableFloat(name + " OutputFrom", 0f, 0f, 1f, false);
            // outputTo = new JSONStorableFloat(name + " Output To", 0f, 0f, 1f, false);
            threshold = new JSONStorableFloat("Threshold", .12f, 0f, 1f, false);
            // SimpleTriggerHandler.LoadAssets();
            exceededTrigger = new EventTrigger(script, "OnExceeded");
            undershootTrigger = new EventTrigger(script, "OnUndershoot");
            valueTrigger = new FloatTrigger(script, "OnValueChanged");
        }

        public void Trigger(float v)
        {
            float delta = inputTo.val - inputFrom.val;
            if (delta > 0f)
            {
                float remappedValue = (v - inputFrom.val) / (inputTo.val - inputFrom.val);
                // float remappedValue = Mathf.Lerp(outputFrom.val, outputTo.val, (v - inputFrom.val) / (inputTo.val - inputFrom.val));
                valueTrigger.Trigger(remappedValue);
            }

            if (_lastValue < threshold.val)
            {
                if(v >= threshold.val) exceededTrigger.Trigger();
            }
            else if(v < threshold.val) undershootTrigger.Trigger();
            // if (_lastValue < threshold.val && v >= threshold.val) exceededTrigger.Trigger();
            // else if (_lastValue >= threshold.val && v < threshold.val) undershootTrigger.Trigger();
            _lastValue = v;
        }

        private void Update()
        {
            exceededTrigger.Update();
            undershootTrigger.Update();
            valueTrigger.Update();
            if (panelOpen) info.val = $"{region} {name}\n{driver.val:0.00}";
            Trigger(driver.val);
        }

        private void OnDestroy()
        {
            exceededTrigger.Remove();
            undershootTrigger.Remove();
            valueTrigger.Remove();
        }

        public void OnAtomRename()
        {
            exceededTrigger.SyncAtomNames();
            undershootTrigger.SyncAtomNames();
            valueTrigger.SyncAtomNames();
        }

        public JSONClass Store(string subScenePrefix)
        {
            JSONClass jc = new JSONClass();
            inputFrom.Store(jc);
            inputTo.Store(jc);
            threshold.Store(jc);
            jc[exceededTrigger.Name] = exceededTrigger.GetJSON(subScenePrefix);
            jc[undershootTrigger.Name] = undershootTrigger.GetJSON(subScenePrefix);
            jc[valueTrigger.Name] = valueTrigger.GetJSON(subScenePrefix);
            return jc;
        }

        public void Load(JSONClass jc, string subScenePrefix)
        {
            if(jc.HasKey(name))
            {
                JSONClass tc = jc[name].AsObject;
                inputFrom.Load(tc);
                inputTo.Load(tc);
                threshold.Load(tc);
                exceededTrigger.RestoreFromJSON(tc, subScenePrefix, false, true);
                undershootTrigger.RestoreFromJSON(tc, subScenePrefix, false, true);
                valueTrigger.RestoreFromJSON(tc, subScenePrefix, false, true);
            }
        }

        public void OpenPanel(MVRScript script, Action back)
        {
            panelOpen = true;
            List<object> triggerUIElements = new List<object>();
            UIDynamicButton button;
            button = script.CreateButton("Return");
            button.buttonColor = new Color(0.55f, 0.90f, 1f);
            button.button.onClick.AddListener(
                () =>
                {
                    Utils.RemoveUIElements(script, triggerUIElements);
                    panelOpen = false;
                    back();
                });
            triggerUIElements.Add(button);
            // var spacer = script.CreateSpacer(true);
            // spacer.height = 50f;
            // triggerUIElements.Add(spacer);

            UIManager.SetupInfoOneLine(triggerUIElements,"Threshold Triggers", false, script: script);

            // Utils.SetupInfoText(script, name, 80f, true);
            var textfield = script.CreateTextField(info, true);
            textfield.height = 80f;
            textfield.UItext.fontSize = 38;
            textfield.UItext.alignment = TextAnchor.MiddleCenter;
            triggerUIElements.Add(textfield);
            
            threshold.CreateUI(triggerUIElements, script: script, rightSide:true);
            button = script.CreateButton("On Threshold Exceeded", false);
            button.button.onClick.AddListener(exceededTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            triggerUIElements.Add(button);
            
            button = script.CreateButton("On Threshold Undershot", false);
            button.button.onClick.AddListener(undershootTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            triggerUIElements.Add(button);

            UIManager.SetupInfoOneLine(triggerUIElements,"Value Triggers", true, script: script);
            // textfield = script.CreateTextField(new JSONStorableString("", "Value Triggers"));
            // textfield.height = 50f;
            // triggerUIElements.Add(textfield);
            // spacer = script.CreateSpacer(true);
            // spacer.height = 55f;
            // triggerUIElements.Add(spacer);
            
            inputFrom.CreateUI(triggerUIElements, script: script);
            inputTo.CreateUI(triggerUIElements, script: script, rightSide:true);
            // outputFrom.CreateUI(triggerUIElements, script: script);
            // outputTo.CreateUI(triggerUIElements, script: script, rightSide:true);
            
            button = script.CreateButton("On Value Changed", false);
            button.button.onClick.AddListener(valueTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            triggerUIElements.Add(button);
        }
    }
}